/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.domain.user;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.yiring.auth.domain.role.Role;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.annotaions.Comment;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.snowflake.SnowflakeId;

/**
 * 用户
 *
 * @author ifzm
 * 2018/9/3 15:27
 */

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Table(
    name = "SYS_USER",
    indexes = {
        @Index(columnList = "mobile", unique = true),
        @Index(columnList = "username", unique = true),
        @Index(columnList = "email", unique = true),
        @Index(columnList = "enabled"),
        @Index(columnList = "deleted"),
    }
)
@Comment("系统用户")
public class User implements Serializable {

    private static final long serialVersionUID = -5787847701210907511L;

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.LONG)
    Long id;

    @Comment("真实姓名")
    String realName;

    @Comment("用户名")
    @Column(unique = true)
    String username;

    @Comment("手机号")
    @Column(unique = true)
    String mobile;

    @Comment("邮箱")
    @Column(unique = true)
    String email;

    @Comment("密码")
    String password;

    @Comment("职称")
    String title;

    @Comment("头像")
    String avatar;

    @Comment("是否启用")
    Boolean enabled;

    @Comment("是否删除")
    Boolean deleted;

    @JsonIgnore
    @Builder.Default
    @Comment("角色集合")
    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(
        name = "SYS_USER_ROLES",
        joinColumns = { @JoinColumn(name = "user_id") },
        inverseJoinColumns = { @JoinColumn(name = "role_id") }
    )
    private Set<Role> roles = new HashSet<>();

    @Comment("最后登录IP地址")
    String lastLoginIp;

    @Comment("最后登录时间")
    LocalDateTime lastLoginTime;

    @Comment("创建时间")
    LocalDateTime createTime;
}
